-----------------------------------------------------------------------------------------------
-- Client Lua Script for CustomFoV
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "Apollo"
require "ChatSystemLib"
 
-----------------------------------------------------------------------------------------------
-- CustomFoV Module Definition
-----------------------------------------------------------------------------------------------
local CustomFoV = {}
local defaultSettings = {}

local userSettings = defaultSettings 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function CustomFoV:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
    return o
end

function CustomFoV:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
	
end

function CustomFoV:OnSave(eLevel)
	-- We save at a character level in case of alts in different guilds
		if (eLevel ~= GameLib.CodeEnumAddonSaveLevel.Account) then
		return
	end

	tSave = {}
	
	self.nFov = nFov
	tSave = {nFov = self.nFov}	
	return tSave
end

function CustomFoV:OnRestore(eLevel,tSavedData)
	if tSavedData.nFov ~= nil then
		userSettings.nFov = tSavedData.nFov
		nFov = tSavedData.nFov
	else
		userSettings.nFov = Apollo.GetConsoleVariable("camera.FovY")
		ChatSystemLib.PostOnChannel(2,"CustomFoV:  Either there was an error in restoring saved FoV settings, or FoV has not been set.  Current FoV: ".. userSettings.nFov)
	end
	
	

end  
-----------------------------------------------------------------------------------------------
-- CustomFoV OnLoad
-----------------------------------------------------------------------------------------------
function CustomFoV:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("CustomFoV.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	
end

-----------------------------------------------------------------------------------------------
-- CustomFoV OnDocLoaded
-----------------------------------------------------------------------------------------------
function CustomFoV:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "CustomFoVForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)
		self.wndTextBox = self.wndMain:FindChild("Label_nFov") -- get strFoVSliderValue Box window
		
		self.wndMain:FindChild("InstructionsFrame"):SetText("Enter your screen size and distance from you to calculate your FoV")
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("setfov", "OnCustomFoVOn", self)
		Apollo.RegisterEventHandler("FovSlider", "OnFovSliderChanged", self)
				
		if type(userSettings.nFov)~= "number" then
			userSettings.nFov = 50
		elseif type(userSettings.nFov) == "number" and nFov ~= nil then
			self.wndTextBox:SetText(tostring(nFov))
		else
			userSettings.nFov = 50
		end

		-- Do additional Addon initialization here
		
		ChatSystemLib.PostOnChannel(2,"CustomFoV:  CustomFoV loaded.  \nUse /setfov to change your settings.")
		
	end
end

-----------------------------------------------------------------------------------------------
-- CustomFoV Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
function CustomFoV:AdjustFoV(strFov)
	nFov = tonumber(strFov)
	if type(nFov) ~= "number" then
		ChatSystemLib.PostOnChannel(2,"CustomFoV: FoV setting must be a number")
		return
		else nFov = self:FovIsValid(nFov)
	end
	Apollo.SetConsoleVariable("camera.FovY", nFov)
	self.wndTextBox:SetText(nFov)
	userSettings.nfov = nFov
	--self.SetSliderValue("Slider_nFov",nFov,false,0)
end

-- on SlashCommand "/setfov"
function CustomFoV:OnCustomFoVOn(cmd, args)
	if type( tonumber( args ) ) == "number" then
		self:AdjustFoV( args )
	else
		--userSettings.nFov = Apollo.GetConsoleVariable("camera.FovY")
		local nAspectWidth = Apollo.GetDisplaySize().nRawWidth
		local nAspectHeight = Apollo.GetDisplaySize().nRawHeight
		self.wndMain:FindChild("Label_nWidth"):SetText(nAspectWidth)
		self.wndMain:FindChild("Label_nHeight"):SetText(nAspectHeight)
		self.wndMain:Invoke() -- show the window
		--self:SetSliderValue("Slider_nFov",userSettings.nFov, false,0)
	end
end

function CustomFoV:FovIsValid(n)
	if n == nil then
		ChatSystemLib.PostOnChannel(2,"CustomFoV:  Default FoV value loaded (50)")
		n = 50
		return n
	elseif n >= 40 and n <= 150 then
		return n
	elseif n < 40 then
		ChatSystemLib.PostOnChannel(2,"CustomFoV:  Values set below 40 may cause problems rendering.  Setting changed to 40.")
		n = 40
		return n
	elseif nFov > 150 then
		ChatSystemLib.PostOnChannel(2,"CustomFoV:  Values set above 150 may cause problems rendering.  Setting changed to 150.")
		n = 150
		return n
	end
end


function CustomFoV:OptimalFieldOfView()
	tFoVVars = { }
	local nDistanceFromScreen = self.wndMain:FindChild("Label_nDistanceToScreen"):GetText()
	local nScreenDiagonal = self.wndMain:FindChild("Label_nScreenDiagonal"):GetText()
	local nAspectWidth = Apollo.GetDisplaySize().nRawWidth/Apollo.GetDisplaySize().nRawHeight
	local nAspectHeight = 1
	local nCalcFoV = self:CalcFoV(nScreenDiagonal,nAspectWidth,nAspectHeight,nDistanceFromScreen)
	table.insert(tFoVVars, 	{["Distance"] = nDistanceFromScreen,["ScreenDiagonal"] = nScreenDiagonal, ["aWidth"] = nAspectWidth,
							["aHeight"] = nAspectHeight,["aDiagonal"] = nAspectDiagonal, ["ScreenWidth"] = nScreenWidth, ["CalcFoV"] = nCalcFoV})
	self.wndTextBox:SetText(nCalcFoV)
	self:OnApply()
  --return math.deg(2 * math.atan2((nScreenWidth / 2), nDistanceFromScreen))

end

--thanks to MrBushido for his help with this fun function
function CustomFoV:CalcFoV(nScreenDiagonal, nAspectWidth, nAspectHeight, nDistanceFromScreen)
  local nAspectDiagonal = math.sqrt(nAspectWidth^2 + nAspectHeight^2)
  local nScreenWidth = nScreenDiagonal / nAspectDiagonal * 16
  return math.floor(math.deg(math.atan2((nScreenWidth / 2), nDistanceFromScreen)) + .5)

end



-----------------------------------------------------------------------------------------------
-- CustomFoVForm Functions
-----------------------------------------------------------------------------------------------
-- when the Cancel button is clicked
function CustomFoV:OnCancel()
	userSettings.nFov = nFov
	self.wndMain:Close() -- hide the window
end

-- when the Apply Changes button is clicked
function CustomFoV:OnApply()
	self:AdjustFoV(self.wndTextBox:GetText())	
end

function CustomFoV:SetSliderValue(sliderName,value,isPercent,roundDigits)
	local nFoVSliderValue = value
	local strFoVSliderValue = value
	nFoVSliderValue = self:round(value, roundDigits)
	strFoVSliderValue = nFoVSliderValue
	self.wndMain:FindChild("Slider_nFov"):SetValue(nFoVSliderValue)
	self.wndMain:FindChild("Label_nFov"):SetText(strFoVSliderValue)
end

function CustomFoV:Slider_BarChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local setting = string.sub(wndControl:GetName(),8) -- setting name is after Slider_
	userSettings[setting] = fNewValue
	self:SetSliderValue(setting,userSettings[setting],false,0)
end

function CustomFoV:round(num, idp)
  	local mult = 10^(idp or 0)
  	return math.floor(num * mult + 0.5) / mult
end

-----------------------------------------------------------------------------------------------
-- CustomFoV Instance
-----------------------------------------------------------------------------------------------
local CustomFoVInst = CustomFoV:new()
CustomFoVInst:Init()
